/*
 *  linux/fs/cse451/file.c
 *
 *  Copyright (C) 1991, 1992 Linus Torvalds
 *
 *  cse451 regular file handling primitives
 */

#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/locks.h>
#include <linux/smp_lock.h>

#include "../cse451fs.h"

typedef u32 block_t;	/* 32 bit, host order */

/*
 * We have mostly NULLs here: the current defaults are OK for
 * the cse451 filesystem.
 */

struct file_operations cse451_file_operations = {
	read:		generic_file_read,
	write:		generic_file_write,
	mmap:		generic_file_mmap,
	fsync:		cse451_sync_file,
};

struct inode_operations cse451_file_inode_operations = {
	truncate:	cse451_truncate,
};

//////////////////////////////////////////////////////////////////
// cse451-only helper functions                                 //
//////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////
// VFS callbacks                                                //
//////////////////////////////////////////////////////////////////

void cse451_truncate(struct inode * inode)
{
	int n;
	long iblock;
	struct cse451_inode_info*	pInodeInfo;

	DEBUGPRINTK (("_truncate\n"))
	DEBUGPRINTK (("...inode = %d i_size = %d\n", (int)inode->i_ino, (int)inode->i_size))

	block_truncate_page(inode->i_mapping, inode->i_size, get_block);

	iblock = (inode->i_size + BLOCK_SIZE-1) >> BLOCK_SIZE_BITS;
	pInodeInfo = (struct cse451_inode_info*)inode->u.generic_ip;

	for (n=iblock; n<CSE451_NUMDATAPTRS; n++) {
		if (pInodeInfo->i_datablocks[n]) {
			DEBUGPRINTK (("_truncate: %d -> %d\n", n, pInodeInfo->i_datablocks[n]))
			cse451_free_block(inode, pInodeInfo->i_datablocks[n]);
			pInodeInfo->i_datablocks[n] = 0;
		}
	}

	inode->i_blocks = 0;

	mark_inode_dirty(inode);

	DEBUGPRINTK (("_truncate: returning...\n"))
}

int cse451_sync_file(struct file * file, struct dentry *dentry, int datasync)
{
        int err = 0;

	DEBUGPRINTK (("_sync_file\n"))

        return (err < 0) ? -EIO : 0;
}

